// Scroll Types
const NORMAL = 1;
const TEXTBOX = 2;

// Pref Types
const BOOL = 0;
const INT  = 1;
const CHAR = 2;

// Global Variables
var scrollbarRegExp = /^xul:(:?slider|thumb)/;
var scrollTarget = null;
var modKeys = new Array('altKey', 'ctrlKey', 'shiftKey', 'metaKey');
var modKey = 0;
var prefs = Components.classes["@mozilla.org/preferences-service;1"].
				getService(Components.interfaces.nsIPrefBranch);

var xsModKey;    // modifier key 0 = alt, 1 = ctrl, 2 = shift, 3 = meta
var xsDelta;     // the amount of movements per action.
var xsDirection; // 0 = up/down => left/right, 1 = up/down => right/left


function xsInit()
{
	xsModKey = getPref("extensions.xscroll.modkey", INT, 2, 0, modKeys.length-1);
	xsDelta = getPref("extensions.xscroll.delta", INT, 36, 1, 360);
	xsDirection = getPref("extensions.xscroll.direction", INT, 0, 0, 1);

	modKey = modKeys[xsModKey];
	window.addEventListener("DOMMouseScroll", xsEventMouseScroll, false);
}

function getPref(aName, aType, aDefault, aMin, aMax)
{
	var retValue = null;
	switch (aType) {
	case BOOL:
		if (prefs.getPrefType(aName) == prefs.PREF_BOOL) {
			retValue = prefs.getBoolPref(aName);
		} else {
			prefs.setBoolPref(aName, aDefault);
			retValue = aDefault;
		}
		break;
	case INT:
		if (prefs.getPrefType(aName) == prefs.PREF_INT) {
			retValue = prefs.getIntPref(aName);
			if (retValue < aMin || retValue > aMax)
				retValue = aDefault;
		} else {
			prefs.setIntPref(aName, aDefault);
			retValue = aDefault;
		}
		break;
	case CHAR:
		if (prefs.getPrefType(aName) == prefs.PREF_STRING) {
			retValue = prefs.getCharPref(aName);
		} else {
			prefs.setCharPref(aName, aDefault);
			retValue = aDefault;
		}
		break;
	}
	return retValue;	
}

function xsEventMouseScroll(aEvent)
{
	if ( (aEvent[modKey] || aEvent.target.focused) && isTextBox(aEvent) ) {
		doXScroll(aEvent, TEXTBOX); 
	} else if ( aEvent[modKey] || isHScrollable(aEvent) ) {
        scrollTarget = aEvent.originalTarget.ownerDocument.defaultView;
		doXScroll(aEvent, NORMAL);
	}
}

function isTextBox(aEvent) {
    if (!aEvent.target.tagName)
		return false;
	else
		var tagName = aEvent.target.tagName.toLowerCase();

    if (tagName == 'textbox') {
		scrollTarget = aEvent.originalTarget;
		return true;
    } else if (tagName == 'textarea') {
		scrollTarget = aEvent.target;
		return true;
    }
}

function isHScrollable(aEvent) {
    if (!aEvent.originalTarget.tagName ||
        aEvent.target.tagName.match(/^textarea/i)) return;

    if (aEvent.originalTarget.tagName.match(scrollbarRegExp) &&
        aEvent.originalTarget.getAttribute("orient") == 'horizontal') {
/*
		if (aEvent.target.tagName.toLowerCase() != 'html' &&
			aEvent.target.ownerDocument.defaultView.getComputedStyle(aEvent.target,"").getPropertyValue('overflow') != 'hidden') {
			scrollTarget = aEvent.target;
			doXScroll(aEvent, TEXTBOX);
			return false;
		}
*/
		return true;
    }
}

function doXScroll(aEvent, aType)
{
	var detail = aEvent.detail;
    if (xsDirection) detail *= -1;
	if (aType == NORMAL) {
		if (detail > 0)
			scrollTarget.scrollBy(xsDelta, 0);
		else
			scrollTarget.scrollBy(-xsDelta, 0);
    } else {
		if (detail > 0)
			scrollTarget.scrollLeft += xsDelta;
		else
			scrollTarget.scrollLeft -= xsDelta;
    }
    aEvent.preventDefault();
}
